Attribute VB_Name = "MVirtualListView"
  Option Private Module
  Option Explicit
  ' demo project showing how to change styles of a VB listview to
  ' create a fully "vurtual list".  a virtual list is a list that
  ' is displayed by a control but the control itself doesn't contian
  ' or "store" the actual list items.  in other words, each time the
  ' control needs to draw an item, it raises an event to get the text
  ' that it needs to draw.  because the control doesn't need to have
  ' each item loaded "into" it, it can display large numbers of items
  ' blindingly fast.  Parts of this project are based on the code
  ' from the blackbelt article by Matt Hart found in the July, 1999
  ' issue of the VBPJ.  Any and all copyrights of the concept and
  ' code presented by Mr. Hart are still in effect and this demo
  ' makes no claim to the contrary.  this extension of Mr. Hart's
  ' concept is provided by Bryan Stafford of New Vision Software -
  ' newvision@mvps.org this demo is released "as is" without
  ' warranty or guaranty of any kind.  In other words, use at your
  ' own risk.
  '
  ' IMPORTANT NOTE:  this demo uses the 'Dbgwproc.dll' from the MS
  ' VB owner's area to allow breaking while running in the VB IDE.
  ' Although you can run this project without using the debug object,
  ' it is STRONGLY recommended that you use it due to the high
  ' potential for crashing the VB IDE without it.  If you choose to
  ' run this project without the debug object, set the Conditional
  ' Compilation Argument 'DEBUGWINDOWPROC' equal to zero on the Make
  ' tab of the Project Properties dialog.  You will also need to
  ' remove the reference to 'Debug Object for AddressOf Subclassing'
  ' in the references dialog.  !!BE WARNED!! If YOU RUN THIS PROJECT
  ' WITHOUT THE DEBUG OBJECT, YOU WILL MOST LIKELY CRASH YOUR
  ' INSTANCE OF VB!!
  '
  '
  ' ALSO NOTE:  this is a minimal implementation of a virtual
  ' listview.  because all of the items are virtual, the internal
  ' listitems collection of the listview is not used.  this means
  ' most of the interaction with the listview must be handled via
  ' the API.  the basic functionality of the listview has been
  ' implemented in this project but many more enhancements are
  ' possible.
  '
  ' this project shows the display of a very large number of items.
  ' it is important to emphasize that displaying large amounts of
  ' data like this is usually a VERY BAD design practice and should
  ' be avoided.  the purpose of this demo is just to show some of
  ' the possibilities that a virtual list opens.
  '
  ' this example project was written using VB5 and the VB5 version
  ' of the listview control.  if you are using VB6, you will need
  ' to change the references to accommodate the VB6 version of the
  ' control.  also, for the VB6 version of the control, you will
  ' need to change the listview class name used in the
  ' VirtualListAppHook procedure from "ListViewWndClass" to
  ' "ListView20WndClass" AND, comment out the line of code specified
  ' in the VirtualListOrStyle_WndProc procedure
  '
  
  
  ' used to keep us from crashing in the IDE.  this variable is set
  ' in the initialize event of the form that hosts the user control
  Public g_bNotDesignMode As Boolean
  
  ' local variables used to set the the hook used to alter
  ' creation time window styles of our combobox
  Private m_hVirtualListHook As Long
  Private m_hHookWndProc As Long

' this function is what we set as the new windowproc when we subclass the usercontrol in the
' usercontrol module.  this function inturn calls the VirtualListFromhWnd function to resolve
' the pointer that we stashed in the the SetProp on the subclassed window
Public Function VirtualListProc(ByVal hWnd&, ByVal uMsg&, ByVal wParam&, ByVal lParam&) As Long
  VirtualListProc = VirtualListFromhWnd(hWnd).VirtualListProc(hWnd, uMsg, wParam, lParam)
End Function

Private Function VirtualListFromhWnd(ByVal hWnd As Long) As UVirtualListView
  ' this function is used to resolve an unreferenced pointer back to
  ' the correct instance of a class object
  
  Dim oObj As UVirtualListView, pObj As Long
    
  pObj = GetProp(hWnd, ByVal "nvVirtualLV")

  CopyMemory oObj, pObj, 4&
  Set VirtualListFromhWnd = oObj
  CopyMemory oObj, 0&, 4&
  
End Function

Public Function LOWORD(ByVal dwValue As Long) As Integer
' Returns the low 16-bit integer from a 32-bit long integer
  CopyMemory LOWORD, dwValue, 2&
End Function

Public Function HIWORD(ByVal nValue&) As Integer
' returns the high 16-bit integer from a 32-bit long integer
  CopyMemory HIWORD, ByVal VarPtr(nValue) + 2, 2&
End Function


' the following portions of this module are based on the code from the blackbelt
' article by Matt Hart found in the July, 1999 issue of the VBPJ.  Any and all
' copyrights of the concept and code presented by Mr. Hart are still in effect
' and this demo makes no claim to the contrary.
Public Function SetVirtualListHook()
  ' set the app hook so that we can intercept the WM_CREATE message for our listbox
  m_hVirtualListHook = SetWindowsHookEx(WH_CALLWNDPROC, AddressOf MVirtualListView.VirtualListAppHook, App.hInstance, App.ThreadID)
End Function

Public Function VirtualListAppHook(ByVal idHook As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
    
  Dim nStrLen&, sClass$, CWP As CWPSTRUCT
  
  ' copy the struct into our local variable
  CopyMemory CWP, ByVal lParam, Len(CWP)
  
  ' check the message
  Select Case CWP.message
    Case WM_CREATE
      ' if it's a WM_CREATE message, get the class name of the control that is being created
      sClass = String(128, 0)
      nStrLen = GetClassName(CWP.hWnd, ByVal sClass, 128)
      sClass = Left$(sClass, nStrLen)

      ' check the class name of the control
      Select Case sClass
        Case "ListViewWndClass", "#32770" ' "ListViewWndClass" is the class name in the IDE and
                                          ' "#32770" is the class name at runtime
            ' note that you must use "ListView20WndClass" for the VB6.0 version of the listview.
        
          ' if it is the listbox, subclass the creator so we can intercept the creation message
          m_hHookWndProc = SetWindowLong(CWP.hWnd, GWL_WNDPROC, AddressOf VirtualListOrStyle_WndProc)
    
          ' we've done what we came here for so unhook the hook....
          Call UnhookWindowsHookEx(m_hVirtualListHook)
            
      End Select
  End Select
  
  ' pass this message on to the next hook
  VirtualListAppHook = CallNextHookEx(m_hVirtualListHook, idHook, wParam, ByVal lParam)

End Function

Public Function VirtualListOrStyle_WndProc(ByVal hWnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
  ' we set this subclass up in the VirtualListAppHook function and now we can actually set
  ' the window styles of the listview to what we want them to be.
  
  ' check for a WM_CREATE message
  Select Case Msg
    Case WM_CREATE
      Dim C As CREATESTRUCT
      
      ' when we get the message, copy the creation struct to our local variable
      CopyMemory C, ByVal lParam, Len(C)
      
      ' then add the styles we want....
      C.style = C.style Or LVS_OWNERDATA
      
      ' copy our altered local struct back to the one that was passed by windows
      CopyMemory ByVal lParam, C, Len(C)
      
      ' alter the styles in the actual window data
      ' (COMMENT OUT THE FOLLOWING LINE FOR THE VB6.0 LISTVIEW)
      Call SetWindowLong(hWnd, GWL_STYLE, C.style)
      
      ' release this subclass since we have accomplished what we came here for
      Call SetWindowLong(hWnd, GWL_WNDPROC, m_hHookWndProc)
  End Select
  
  ' pass all messages on to the default windowproc
  VirtualListOrStyle_WndProc = CallWindowProc(m_hHookWndProc, hWnd, Msg, wParam, lParam)
  
End Function


